@@ -59,12 +59,12 @@ from math import log, log10, sqrt, pi, ceil, floor
 
 from numpy import inf, isinf, isfinite, clip
 import numpy.random as RNG
-
 try:
     from scipy.stats import norm as normal_distribution
 except ImportError:
-    def normal_distribution(*args, **kw):
-        raise RuntimeError("scipy.stats unavailable")
+    # Normal distribution is an optional dependency.  Leave it as a runtime
+    # failure if it doesn't exist.
+    pass
 
 
 def pm(v, *args):
@@ -194,7 +194,6 @@ def init_bounds(v):
 
 
 class Bounds(object):
-
     """
     Bounds abstract base class.
 
@@ -289,6 +288,12 @@ class Bounds(object):
         limits = tuple(num_format(v) for v in self.limits)
         return "(%s,%s)" % limits
 
+    def to_dict(self):
+        return dict(
+            type=type(self).__name__,
+            limits=self.limits,
+            )
+
 # CRUFT: python 2.5 doesn't format indefinite numbers properly on windows
 
 
@@ -305,7 +310,6 @@ def num_format(v):
 
 
 class Unbounded(Bounds):
-
     """
     Unbounded parameter.
 
@@ -317,7 +321,7 @@ class Unbounded(Bounds):
     """
 
     def random(self, n=1, target=1.0):
-        scale = target + (target==0.)
+        scale = target + (target == 0.)
         return RNG.randn(n)*scale
 
     def nllf(self, value):
@@ -340,7 +344,6 @@ class Unbounded(Bounds):
 
 
 class BoundedBelow(Bounds):
-
     """
     Semidefinite range bounded below.
 
@@ -368,7 +371,7 @@ class BoundedBelow(Bounds):
 
     def random(self, n=1, target=1.):
         target = max(abs(target), abs(self._base))
-        scale = target + (target==0.)
+        scale = target + (target == 0.)
         return self._base + abs(RNG.randn(n)*scale)
 
     def nllf(self, value):
@@ -402,7 +405,6 @@ class BoundedBelow(Bounds):
 
 
 class BoundedAbove(Bounds):
-
     """
     Semidefinite range bounded above.
 
@@ -428,7 +430,7 @@ class BoundedAbove(Bounds):
 
     def random(self, n=1, target=1.0):
         target = max(abs(self._base), abs(target))
-        scale = target + (target==0.)
+        scale = target + (target == 0.)
         return self._base - abs(RNG.randn(n)*scale)
 
     def nllf(self, value):
@@ -462,7 +464,6 @@ class BoundedAbove(Bounds):
 
 
 class Bounded(Bounds):
-
     """
     Bounded range.
 
@@ -509,7 +510,6 @@ class Bounded(Bounds):
 
 
 class Distribution(Bounds):
-
     """
     Parameter is pulled from a distribution.
 
@@ -554,9 +554,16 @@ class Distribution(Bounds):
         return "%s(%s)" % (self.dist.dist.name,
                            ",".join(str(s) for s in self.dist.args))
 
+    def to_dict(self):
+        return dict(
+            type=type(self).__name__,
+            limits=self.limits,
+            # TODO: how to handle arbitrary distribution function in save/load?
+            dist=type(self.dist).__name__,
+            )
+
 
 class Normal(Distribution):
-
     """
     Parameter is pulled from a normal distribution.
 
@@ -593,7 +600,6 @@ class Normal(Distribution):
 
 
 class BoundedNormal(Bounds):
-
     """
     truncated normal bounds
     """
@@ -682,7 +688,6 @@ class BoundedNormal(Bounds):
 
 
 class SoftBounded(Bounds):
-
     """
     Parameter is pulled from a stretched normal distribution.
 
