@@ -72,6 +72,7 @@ from __future__ import division, print_function
 
 __all__ = ['MCMCDraw', 'load_state', 'save_state']
 
+import os.path
 import re
 import gzip
 
@@ -152,7 +153,7 @@ def save_state(state, filename):
     write(trace, "writing point\n")
     fid = CREATE(filename+'-point'+EXT, 'wb')
     write(fid, '# logp point (Nthin x Npop x Nvar = [%d,%d,%d])\n'
-               % (Nthin, Npop, Nvar))
+          % (Nthin, Npop, Nvar))
     savetxt(fid, point)
     fid.close()
 
@@ -187,7 +188,7 @@ def loadtxt(file, report=0):
     lineno = 0
     for line in fh:
         lineno += 1
-        if report and lineno%report==0:
+        if report and lineno%report == 0:
             print("read", section*report)
             section += 1
         IND_PAT.sub('nan', line)
@@ -204,7 +205,11 @@ def loadtxt(file, report=0):
     return asarray(res)
 
 
-def load_state(filename, skip=0, report=0):
+def path_contains_saved_state(filename):
+    chain_file = filename + '-chain' + EXT
+    return os.path.exists(chain_file)
+
+def load_state(filename, skip=0, report=0, derived_vars=0):
     # Read chain file
     chain = loadtxt(filename+'-chain'+EXT)
 
@@ -242,18 +247,18 @@ def load_state(filename, skip=0, report=0):
     state._thin_index = 0
     state._thin_draws = state._gen_draws[(skip+1)*thinning-1::thinning]
     state._thin_logp = point[:, 0].reshape((Nthin, Npop))
-    state._thin_point = reshape(point[:, 1:], (Nthin, Npop, Nvar))
-    state._gen_current = state._thin_point[-1]
+    state._thin_point = reshape(point[:, 1:Nvar+1-derived_vars], (Nthin, Npop, -1))
+    state._gen_current = state._thin_point[-1].copy()
     state._update_count = Nupdate
     state._update_index = 0
     state._update_draws = stats[:, 0]
-    state._update_R_stat = stats[:, 1:Nvar+1]
-    state._update_CR_weight = stats[:, Nvar+1:]
+    state._update_R_stat = stats[:, 1:Nvar+1-derived_vars]
+    state._update_CR_weight = stats[:, Nvar+1-derived_vars:]
     state._outliers = []
 
     bestidx = np.argmax(point[:, 0])
     state._best_logp = point[bestidx, 0]
-    state._best_x = point[bestidx, 1:]
+    state._best_x = point[bestidx, 1:Nvar+1-derived_vars]
 
     return state
 
@@ -262,11 +267,8 @@ class MCMCDraw(object):
     """
     """
     _labels = None
+    _integer_vars = None  # boolean array of integer variables, or None
     title = None
-    @property
-    def Nvar(self):
-        """Number of parameters in the fit"""
-        return self._thin_point.shape[2]
 
     def __init__(self, Ngen, Nthin, Nupdate, Nvar, Npop, Ncr, thinning):
         # Total number of draws so far
@@ -280,7 +282,7 @@ class MCMCDraw(object):
         self.generation = 0
         self._gen_index = 0
         self._gen_draws = empty(Ngen, 'i')
-        self._gen_logp = empty( (Ngen, Npop) )
+        self._gen_logp = empty((Ngen, Npop))
         self._gen_acceptance_rate = empty(Ngen)
 
         # If we are thinning, we need to keep the current generation
@@ -303,7 +305,7 @@ class MCMCDraw(object):
         self._update_index = 0
         self._update_count = 0
         self._update_draws = empty(Nupdate, 'i')
-        self._update_R_stat = empty((Nupdate, Nvar) )
+        self._update_R_stat = empty((Nupdate, Nvar))
         self._update_CR_weight = empty((Nupdate, Ncr))
 
         self._outliers = []
@@ -327,6 +329,7 @@ class MCMCDraw(object):
 
     @property
     def Nvar(self):
+        """Number of parameters in the fit"""
         return self._thin_point.shape[2]
 
     @property
@@ -355,7 +358,7 @@ class MCMCDraw(object):
         if Ngen > self.Ngen:
             self._gen_index = self.Ngen  # must happen before resize!!
             self._gen_draws = np.resize(self._gen_draws, Ngen)
-            self._gen_logp = np.resize(self._gen_logp,  (Ngen, Npop))
+            self._gen_logp = np.resize(self._gen_logp, (Ngen, Npop))
             self._gen_acceptance_rate \
                 = np.resize(self._gen_acceptance_rate, Ngen)
         elif Ngen < self.Ngen:
@@ -367,8 +370,8 @@ class MCMCDraw(object):
         if Nthin > self.Nthin:
             self._thin_index = self.Nthin  # must happen before resize!!
             self._thin_draws = np.resize(self._thin_draws, Nthin)
-            self._thin_point = np.resize(self._thin_point,  (Nthin, Npop, Nvar))
-            self._thin_logp = np.resize(self._thin_logp,  (Nthin, Npop))
+            self._thin_point = np.resize(self._thin_point, (Nthin, Npop, Nvar))
+            self._thin_logp = np.resize(self._thin_logp, (Nthin, Npop))
         elif Nthin < self.Nthin:
             self._thin_draws = self._thin_draws[-Nthin:].copy()
             self._thin_point = self._thin_point[-Nthin:, :, :].copy()
@@ -378,9 +381,9 @@ class MCMCDraw(object):
             self._update_count = self.Nupdate  # must happen before resize!!
             self._update_draws = np.resize(self._update_draws, Nupdate)
             self._update_R_stat \
-                = np.resize(self._update_R_stat,  (Nupdate, Nvar))
+                = np.resize(self._update_R_stat, (Nupdate, Nvar))
             self._update_CR_weight \
-                = np.resize(self._update_CR_weight,  (Nupdate, Ncr))
+                = np.resize(self._update_CR_weight, (Nupdate, Ncr))
         elif Nupdate < self.Nupdate:
             self._update_draws = self._update_draws[-Nupdate:].copy()
             self._update_R_stat = self._update_R_stat[-Nupdate:, :].copy()
@@ -441,7 +444,8 @@ class MCMCDraw(object):
             self._thin_point[i] = x
             self._thin_logp[i] = logp
             i = i+1
-            if i == len(self._thin_draws): i = 0
+            if i == len(self._thin_draws):
+                i = 0
             self._thin_index = i
             self._gen_current = x+0 # force a copy
         else:
@@ -528,7 +532,7 @@ class MCMCDraw(object):
             # numbers generated at or above the cursor.  All of these must
             # be shifted by Nchains to avoid the cursor.
             perm = draw(Npop-Nchain, pool_size)
-            perm[perm>=cursor] += Nchain
+            perm[perm >= cursor] += Nchain
             #print("perm", perm; raw_input('wait'))
             pop[Nchain:] = points[perm]
 
@@ -781,7 +785,7 @@ class MCMCDraw(object):
 
         # Find the location of the best point if it exists and swap with
         # the final position
-        idx = np.where(logp==self._best_logp)[0]
+        idx = np.where(logp == self._best_logp)[0]
         if len(idx) == 0:
             logp[final] = self._best_logp
             points[final, :] = self._best_x
@@ -804,36 +808,96 @@ class MCMCDraw(object):
         drawn = self.draw(**kw)
         return drawn.points, drawn.logp
 
-    def entropy(self, **kw):
+    def entropy(self, vars=None, portion=1, selection=None, n_est=10000, thin=None):
         """
         Return entropy estimate and uncertainty from an MCMC draw.
 
-        See :func:`entropy.entropy` for details.
+        *portion* is the portion of each chain to use
+
+        *vars* is the set of variables to marginalize over.  It is None for
+        the visible variables, or a list of variables.
+
+        *vars* is the list of variables to use for marginalization.
+
+        *selection* sets the range each parameter in the returned distribution,
+        using {variable: (low, high)}. Missing variables use the full range.
+
+        *n_est* is the number of points to use from the draw when estimating
+        the entropy (default=10000).
+
+        *thin* is the amount of thinning to use when selecting points from the
+        draw.
         """
-        from .entropy import entropy, MVNEntropy
+        from . import entropy
+        from .entropy import Timer as T
 
-        # Get the sample from the state
-        drawn = self.draw()
+        # Get the sample from the state.
+        # set default thinning to max((steps * samples/step) // n_est, 1)
+        if thin is None:
+            Nsteps = min(self.Nthin, self._thin_count)
+            thin = max(Nsteps*self.Npop//n_est, 1)
+            #print("thin", thin, Nsteps, self.Npop, self.Nthin, self._thin_count)
+        drawn = self.draw(portion=portion, vars=vars,
+                          selection=selection, thin=thin)
 
-        M = MVNEntropy(drawn.points)
+        # TODO: don't print within a library function!
+        #with T():
+        M = entropy.MVNEntropy(drawn.points)
         print("Entropy from MVN: %s"%str(M))
-        if M.reject_normal:
-            return entropy(drawn.points, drawn.logp, **kw)
-        else:
-            return M.entropy, 0
 
+        # Try wnn . . . no good.
+        #with T(): S_wnn, Serr_wnn = entropy.wnn_entropy(drawn.points, n_est=20000)
+        #print("Entropy from wnn: %s"%str(S_wnn))
 
-    def draw(self, portion=1, vars=None, selection=None):
+        # Try wnn with bootstrap . . . still no good.
+        #with T(): S_wnn, Serr_wnn = entropy.wnn_bootstrap(drawn.points)
+        #print("Entropy from wnn bootstrap: %s"%str(S_wnn))
+
+        # Try wnn entropy with thinning . . . still no good.
+        #drawn = self.draw(portion=portion, vars=vars,
+        #                  selection=selection, thin=10)
+        #with T(): S_wnn, Serr_wnn = entropy.wnn_entropy(points)
+        #print("Entropy from wnn: %s"%str(S_wnn))
+
+        # Try wnn with gmm ... still no good
+        #with T(): S_wnn, Serr_wnn = entropy.wnn_entropy(drawn.points, n_est=20000, gmm=20)
+        #print("Entropy from wnn with gmm: %s"%str(S_wnn))
+
+        # Try pure gmm ... pretty good
+        #with T(): S_gmm, Serr_gmm = entropy.gmm_entropy(drawn.points, n_est=10000)
+        #print("Entropy from gmm: %s"%str(S_gmm))
+
+        # Try kde from statsmodels ... pretty good
+        #with T(): S_kde_stats = entropy.kde_entropy_statsmodels(drawn.points, n_est=10000)
+        #print("Entropy from kde statsmodels: %s"%str(S_kde_stats))
+
+        # Try kde from sklearn ... pretty good
+        #with T(): S_kde = entropy.kde_entropy_sklearn(drawn.points, n_est=10000)
+        #print("Entropy from kde sklearn: %s"%str(S_kde))
+
+        # Try kde from sklearn at points from gmm ... pretty good
+        #with T(): S_kde_gmm = entropy.kde_entropy_sklearn_gmm(drawn.points, n_est=10000)
+        #print("Entropy from kde+gmm: %s"%str(S_kde_gmm))
+
+        #with T():
+        S, Serr = entropy.entropy(drawn.points, drawn.logp, N_entropy=n_est)
+        #print("Entropy from Kramer: %s"%str(S))
+
+        # Always return entropy estimate from draw, even if it is normal
+        return S, Serr
+
+    def draw(self, portion=1, vars=None, selection=None, thin=1):
         """
         Return a sample from the posterior distribution.
 
         *portion* is the portion of each chain to use
-        *vars* is a list of variables to return for each point
-        *selection* sets the range for the returned marginal distribution
 
-        *selection* is a dictionary of {variable: (low, high)} to set the
-        range on each variable.  Missing variables default to the full
-        range.
+        *vars* is a list of variables to return for each point
+
+        *selection* sets the range each parameter in the returned distribution,
+        using {variable: (low, high)}. Missing variables use the full range.
+
+        *thin* takes every nth item.
 
         To plot the distribution for parameter p1::
 
@@ -846,7 +910,8 @@ class MCMCDraw(object):
             plot(draw.points[:, 0], draw.points[:, 1], '.')
         """
         vars = vars if vars is not None else getattr(self, '_shown', None)
-        return Draw(self, portion=portion, vars=vars, selection=selection)
+        return Draw(self, portion=portion, vars=vars, selection=selection,
+                    thin=thin)
 
     def set_visible_vars(self, labels):
         self._shown = [self.labels.index(v) for v in labels]
@@ -854,20 +919,12 @@ class MCMCDraw(object):
         #print(labels)
         #print(self._shown)
 
-    def integer_vars(self, labels):
+    def set_integer_vars(self, labels):
         """
-        Set variables to integer variables by rounding their values to the
-        nearest integer.
-
-        Note that this cannot be done ahead of time unless DREAM gets an
-        integer stepper, but it can be done before generating statistics.
-        DREAM on the OpenBUGS Asia model does not return the same results
-        as OpenBUGS, so the analysis of integer parameters should not yet
-        be trusted.  No other tests have been done to this point.
+        Indicate tha variables should be considered integer variables when
+        computing statistics.
         """
-        for var in labels:
-            idx = self.labels.index(var)
-            self._thin_point[:,:,idx] = np.round(self._thin_point[:,:,idx])
+        self._integer_vars = np.array([var in labels for var in self.labels])
 
     def derive_vars(self, fn, labels=None):
         """
@@ -917,21 +974,25 @@ class MCMCDraw(object):
 
 
 class Draw(object):
-    def __init__(self, state, vars=None, portion=None, selection=None):
+    def __init__(self, state, vars=None, portion=None, selection=None, thin=1):
         self.state = state
         self.vars = vars
         self.portion = portion
         self.selection = selection
-        self.points, self.logp \
-            = _sample(state, portion=portion, vars=vars, selection=selection)
+        self.points, self.logp = _sample(
+            state, portion=portion, vars=vars, selection=selection, thin=thin)
         self.labels \
             = state.labels if vars is None else [state.labels[v] for v in vars]
         self._stats = None
         self.weights = None
         self.num_vars = len(self.labels)
+        if state._integer_vars is not None:
+            self.integers = state._integer_vars[vars] if vars else None
+        else:
+            self.integers = None
 
 
-def _sample(state, portion, vars, selection):
+def _sample(state, portion, vars, selection, thin):
     """
     Return a sample from a set of chains.
     """
@@ -939,19 +1000,19 @@ def _sample(state, portion, vars, selection):
     start = int((1-portion)*len(draw)) if portion else 0
 
     # Collect the subset we are interested in
-    chains = chains[start:, state._good_chains, :]
-    logp = logp[start:, state._good_chains]
+    chains = chains[start::thin, state._good_chains, :]
+    logp = logp[start::thin, state._good_chains]
 
     Ngen, Npop, Nvar = chains.shape
-    points = reshape(chains, (Ngen*Npop, Nvar))
-    logp = reshape(logp, (Ngen*Npop))
+    points = reshape(chains, (-1, Nvar))
+    logp = reshape(logp, (-1))
     if selection not in [None, {}]:
         idx = True
         for v, r in selection.items():
             if v == 'logp':
-                idx = idx & (logp>=r[0]) & (logp<=r[1])
+                idx = idx & (logp >= r[0]) & (logp <= r[1])
             else:
-                idx = idx & (points[:, v]>=r[0]) & (points[:, v]<=r[1])
+                idx = idx & (points[:, v] >= r[0]) & (points[:, v] <= r[1])
         points = points[idx, :]
         logp = logp[idx]
     if vars is not None:
