@@ -74,13 +74,16 @@ class Fitness(object):
     """
     def parameters(self):
         """
-        Return the parameters in the model.
+        return the parameters in the model.
 
-        Model parameters are a hierarchical structure of lists and
+        model parameters are a hierarchical structure of lists and
         dictionaries.
         """
         raise NotImplementedError()
 
+    def to_dict(self):
+        raise NotImplementedError()
+
     def update(self):
         """
         Called when parameters have been updated.  Any cached values will need
@@ -256,6 +259,18 @@ class BaseFitProblem(object):
         """
         return self.fitness.parameters()
 
+    def to_dict(self):
+        return {
+            'type': type(self).__name__,
+            'name': self.name,
+            'fitness': self.fitness.to_dict(),
+            'partial': self.partial,
+            'soft_limit': self.soft_limit,
+            'penalty_nllf': self.penalty_nllf,
+            # TODO: constraints may be a function.
+            'constraints': str(self.constraints),
+        }
+
     def model_points(self):
         """
         Number of data points associated with the model.
@@ -628,6 +643,20 @@ class MultiFitProblem(BaseFitProblem):
             pars['freevars'] = free
         return pars
 
+    def to_dict(self):
+        return {
+            'type': type(self).__name__,
+            'name': self.name,
+            'models': [p.to_dict() for p in self._models],
+            'weights': self.weights,
+            'partial': self.partial,
+            'soft_limit': self.soft_limit,
+            'penalty_nllf': self.penalty_nllf,
+            # TODO: constraints may be a function.
+            'constraints': str(self.constraints),
+            'freevars': self.freevars.to_dict(),
+        }
+
     def model_points(self):
         """Return number of points in all models"""
         return sum(f.model_points() for f in self.models)
