@@ -4,8 +4,14 @@ Interfaces to various optimizers.
 from __future__ import print_function, division
 
 import sys
-import time
 from copy import copy
+import warnings
+
+# CRUFT: time.clock() removed from python 3.8
+try:
+    from time import perf_counter
+except ImportError:
+    from time import clock as perf_counter
 
 import numpy as np
 
@@ -36,7 +42,7 @@ class ConsoleMonitor(monitor.TimedUpdate):
         sys.stdout.flush()
 
     def show_improvement(self, history):
-        # print "step",history.step[0],"chisq",history.value[0]
+        # print("step",history.step[0],"chisq",history.value[0])
         p = self.problem.getp()
         try:
             self.problem.setp(history.point[0])
@@ -46,6 +52,30 @@ class ConsoleMonitor(monitor.TimedUpdate):
         sys.stdout.flush()
 
 
+class CheckpointMonitor(monitor.TimedUpdate):
+    """
+    Periodically save fit state so that it can be resumed later.
+    """
+    #: Function to call at each checkpoint.
+    checkpoint = None  # type: Callable[None, None]
+    def __init__(self, checkpoint, progress=60*30):
+        monitor.TimedUpdate.__init__(self, progress=progress,
+                                     improvement=np.inf)
+        self.checkpoint = checkpoint
+        self._first = True
+
+    def show_progress(self, history):
+        # Skip the first checkpoint since it only contains the
+        # start/resume state
+        if self._first:
+            self._first = False
+        else:
+            self.checkpoint(history)
+
+    def show_improvement(self, history):
+        pass
+
+
 class StepMonitor(monitor.Monitor):
     """
     Collect information at every step of the fit and save it to a file.
@@ -91,11 +121,11 @@ class MonitorRunner(object):
                                population_points=1, population_values=1)
         for M in self.monitors:
             M.config_history(self.history)
-        self._start = time.time()
+        self._start = perf_counter()
 
     def __call__(self, step, point, value,
                  population_points=None, population_values=None):
-        self.history.update(time=time.time() - self._start,
+        self.history.update(time=perf_counter() - self._start,
                             step=step, point=point, value=value,
                             population_points=population_points,
                             population_values=population_values)
@@ -476,7 +506,7 @@ class SimplexFit(FitBase):
         # TODO: no mapper??
         self._update = MonitorRunner(problem=self.problem,
                                      monitors=monitors)
-        # print "bounds",self.problem.bounds()
+        #print("bounds", self.problem.bounds())
         result = simplex(f=self.problem.nllf, x0=self.problem.getp(),
                          bounds=self.problem.bounds(),
                          abort_test=abort_test,
@@ -795,7 +825,7 @@ class DreamFit(FitBase):
 
     def _monitor(self, state, pop, logp):
         # Get an early copy of the state
-        self._update.history.uncertainty_state = state
+        self.state = self._update.history.uncertainty_state = state
         step = state.generation
         x, fx = state.best()
         self._update(step=step, point=x, value=-fx,
@@ -823,16 +853,9 @@ class DreamFit(FitBase):
             print("loading saved state (this might take awhile) ...")
             fn, labels = getattr(self.problem, 'derive_vars', (None, []))
             self.state = load_state(input_path, report=100, derived_vars=len(labels))
-        elif input_path != problem.output_path:
-            # Only signal an error if --resume is different from --store.  If
-            # it is the same (e.g., because "--resume=-" was given on the
-            # command line) but missing, then silently skip the resume.  Note
-            # that we can't just check for the existence of the directory
-            # since we will already have created it before we get here.  We
-            # do want to signal an error if resume is different from store
-            # in order to catch typos on the command line instead of letting
-            # users quietly believe they are resuming.
-            raise RuntimeError("no mcmc saved at %r"%input_path)
+        else:
+            # Warn if mc files are not found on --resume path
+            warnings.warn("No mcmc found; ignoring --resume=%r"%input_path)
 
     def save(self, output_path):
         self.state.save(output_path)
@@ -933,13 +956,13 @@ class FitDriver(object):
         starts = self.options.get('starts', 1)
         if starts > 1:
             fitter = MultiStart(fitter)
-        t0 = time.clock()
+        t0 = perf_counter()
         self.fitter = fitter
         x, fx = fitter.solve(monitors=self.monitors,
                              abort_test=self.abort_test,
                              mapper=self.mapper,
                              **self.options)
-        self.time = time.clock() - t0
+        self.time = perf_counter() - t0
         self.result = x, fx
         if x is not None:
             self.problem.setp(x)
@@ -1123,31 +1146,44 @@ def _fill_defaults(options, settings):
     result.update(options)
     return result
 
-# List of (parameter,factory value) required for each algorithm
-FITTERS = [
-    SimplexFit,
-    DEFit,
-    DreamFit,
-    BFGSFit,
-    LevenbergMarquardtFit,
-    MPFit,
-    PSFit,
-    PTFit,
-    RLFit,
-    SnobFit,
-    ]
 
-FIT_AVAILABLE_IDS = [f.id for f in FITTERS]
+FITTERS = []
+FIT_AVAILABLE_IDS = []
+FIT_ACTIVE_IDS = []
+def register(fitter, active=True):
+    """
+    Register a new fitter with bumps, if it is not already there.
 
+    *active* is False if you don't want it showing up in the GUI selector.
+    """
+    # Check if already registered.
+    if fitter in FITTERS:
+        return
 
-FIT_ACTIVE_IDS = [
-    SimplexFit.id,
-    DEFit.id,
-    DreamFit.id,
-    BFGSFit.id,
-    LevenbergMarquardtFit.id,
-    MPFit.id,
-    ]
+    # Check that there is no other fitter of that name
+    if fitter.id in FIT_AVAILABLE_IDS:
+        raise ValueError("There is already a fitter registered as %r"
+                         % fitter.id)
+
+    # Register the fitter.
+    FITTERS.append(fitter)
+    FIT_AVAILABLE_IDS.append(fitter.id)
+
+    # Make it "active" by listing it in the help menu.
+    if active:
+        FIT_ACTIVE_IDS.append(fitter.id)
+
+# Register the fitters
+register(SimplexFit, active=True)
+register(DEFit, active=True)
+register(DreamFit, active=True)
+register(BFGSFit, active=True)
+register(LevenbergMarquardtFit, active=True)
+register(MPFit, active=True)
+register(PSFit, active=False)
+register(PTFit, active=False)
+register(RLFit, active=False)
+register(SnobFit, active=False)
 
 FIT_DEFAULT_ID = SimplexFit.id
 
@@ -1202,4 +1238,4 @@ def fit(problem, method=FIT_DEFAULT_ID, verbose=False, **options):
         )
     if hasattr(driver.fitter, 'state'):
         result.state = driver.fitter.state
-    return result
+    return result
\ No newline at end of file
