@@ -255,6 +255,19 @@ class BaseParameter(object):
     def __repr__(self):
         return "Parameter(%s)" % self
 
+    def to_dict(self):
+        """
+        Return a dict represention of the object.
+        """
+        return dict(
+            type=type(self).__name__,
+            name=self.name,
+            value=self.value,
+            fixed=self.fixed,
+            fittable=self.fittable,
+            bounds=self._bounds.to_dict(),
+            )
+
 
 class Constant(BaseParameter):
     """
@@ -567,6 +580,15 @@ class FreeVariables(object):
         """
         return dict((k, v.parameters) for k, v in self._parametersets.items())
 
+    def to_dict(self):
+        return {
+            'type': type(self).__name__,
+            'names': self.names,
+            'parameters': {
+                k: v.to_dict() for k, v in self._parametersets.items()
+            }
+        }
+
     def set_model(self, i):
         """
         Set the reference parameters for model *i*.
@@ -733,6 +755,7 @@ class Function(BaseParameter):
 
     def __setstate__(self, state):
         self.name, self.op, self.args, self.kw = state
+        self._parameters = self._find_parameters()
 
     def __str__(self):
         if self.name is not None:
@@ -1039,3 +1062,12 @@ class Alias(object):
 
     def parameters(self):
         return self.p.parameters()
+
+    def to_dict(self):
+        return {
+            'type': type(self).__name__,
+            'p': self.p.to_dict(),
+            # TODO: can't json pickle arbitrary objects
+            'obj': str(self.obj),
+            'attr': self.attr,
+        }
